@@ -2,29 +2,58 @@
 # CodeWriter21
 
 import io as _io
+import logging as _logging
 
 from typing import Union as _Union, Dict as _Dict
 
 from log21.Levels import *
 from log21.Logger import Logger
 from log21.Manager import Manager
+from log21.ProgressBar import ProgressBar
 from log21.PPrint import PrettyPrinter, pformat
 from log21.TreePrint import TreePrint, tree_format
 from log21.Argparse import ColorizingArgumentParser
 from log21.FileHandler import DecolorizingFileHandler
+from log21.LoggingWindow import LoggingWindow, LoggingWindowHandler
 from log21.StreamHandler import ColorizingStreamHandler, StreamHandler
 from log21.Formatters import ColorizingFormatter, DecolorizingFormatter
-from log21.Colors import Colors, get_color, get_colors, ansi_esc, get_color_name, closest_color
+from log21.Colors import Colors, get_color, get_colors, ansi_escape, get_color_name, closest_color
 
-__version__ = "1.5.8"
+__version__ = "2.1.2"
 __author__ = "CodeWriter21 (Mehrad Pooryoussof)"
 __github__ = "Https://GitHub.com/MPCodeWriter21/log21"
 __all__ = ['ColorizingStreamHandler', 'DecolorizingFileHandler', 'ColorizingFormatter', 'DecolorizingFormatter',
            'get_logger', 'Logger', 'Colors', 'get_color', 'get_colors', 'CRITICAL', 'FATAL', 'ERROR', 'WARNING', 'WARN',
            'INFO', 'DEBUG', 'NOTSET', 'StreamHandler', 'ColorizingArgumentParser', 'PrettyPrinter', 'pformat',
-           'pprint', 'pretty_print']
+           'pprint', 'pretty_print', 'tree_format', 'TreePrint', 'Manager', 'get_color_name', 'closest_color',
+           'ansi_escape', '__version__', '__author__', '__github__', 'debug', 'info', 'warning', 'warn', 'error',
+           'critical', 'fatal', 'exception', 'log', 'basic_config', 'basicConfig', 'ProgressBar', 'progress_bar',
+           'LoggingWindow', 'LoggingWindowHandler', 'get_logging_window']
 
 _manager = Manager()
+_logging.setLoggerClass(Logger)
+
+
+def _prepare_formatter(fmt: str = None, style: str = '%', datefmt: str = "%H:%M:%S", show_level: bool = True,
+                       show_time: bool = True, colorize_time_and_level: bool = True,
+                       level_names: _Dict[int, str] = None):
+    # Prepares a formatting if the fmt was None
+    if not fmt:
+        style = '%'
+        fmt = "%(message)s"
+        if show_level:
+            fmt = "[%(levelname)s] " + fmt
+        if show_time:
+            fmt = "[%(asctime)s] " + fmt
+        fmt = '\r' + fmt
+    # Defines the formatter
+    formatter = ColorizingFormatter(fmt, datefmt, style=style, level_names=level_names)
+    if not colorize_time_and_level:
+        for key in formatter.level_colors:
+            formatter.level_colors[key] = tuple()
+        formatter.time_color = tuple()
+
+    return formatter
 
 
 def get_logger(name: str = '', level: _Union[int, str] = NOTSET, show_time: bool = True,
@@ -87,8 +116,7 @@ def get_logger(name: str = '', level: _Union[int, str] = NOTSET, show_time: bool
     :param handle_new_line: bool = True: Places the NewLine characters at the beginning of the text before everything else
     :param override: bool = True: Overrides the logger attributes even if it already exists
     :param level_names: Dict[int, str] = None: You can specify custom level names.
-    :return: logging.Logger
-
+    :return: log21.Logger
     """
     if not isinstance(name, str):
         raise TypeError('A logger name must be a string')
@@ -97,20 +125,7 @@ def get_logger(name: str = '', level: _Union[int, str] = NOTSET, show_time: bool
         logger = _manager.getLogger(name)
     if (not logger) or override:
         logger = Logger(name, level)
-        # Prepares a formatting if the fmt was None
-        if not fmt:
-            fmt = "%(message)s"
-            if show_level:
-                fmt = "[%(levelname)s] " + fmt
-            if show_time:
-                fmt = "[%(asctime)s] " + fmt
-            fmt = '\r' + fmt
-        # Defines the formatter
-        formatter = ColorizingFormatter(fmt, datefmt, style=style, level_names=level_names)
-        if not colorize_time_and_level:
-            for key in formatter.level_colors:
-                formatter.level_colors[key] = tuple()
-            formatter.time_color = tuple()
+        formatter = _prepare_formatter(fmt, style, datefmt, show_level, show_time, colorize_time_and_level, level_names)
 
         # Defines the handler
         handler = ColorizingStreamHandler(handle_carriage_return=handle_carriage_return,
@@ -121,6 +136,79 @@ def get_logger(name: str = '', level: _Union[int, str] = NOTSET, show_time: bool
     return logger
 
 
+def get_logging_window(name: str = '', level: _Union[int, str] = NOTSET, show_time: bool = True,
+                       show_level: bool = True, colorize_time_and_level: bool = True, fmt: str = None,
+                       datefmt: str = "%H:%M:%S", style: str = '%', handle_carriage_return: bool = True,
+                       handle_new_line: bool = True, override=False, level_names: _Dict[int, str] = None,
+                       width: int = 80, height: int = 20, allow_shell: bool = False) -> LoggingWindow:
+    """
+    Returns a logging window.
+
+    >>> # Let's see how it works
+    >>> # Imports log21 and time modules
+    >>> import log21, time
+    >>> # Creates a new LoggingWindow object
+    >>> window = log21.get_logging_window('Test Window')
+    >>> # Now use it without any additional steps to add handlers and formatters!
+    >>> # It works just like a normal logger but with some extra features
+    >>>
+    >>> window.info('This works properly!')
+    >>>
+    >>> # You can use HEX colors as well as the ANSI colors which are supported by normal loggers
+    >>> # ANSI colors usage:
+    >>> window.info('This is a \033[91mred\033[0m message.')
+    >>> window.info('\033[102mThis is a message with green background.')
+    >>> # HEX colors usage:
+    >>> window.info('\033#00FFFFhfThis is a message with cyan foreground.')
+    >>> window.info('\033#0000FFhbThis is a message with blue background.')
+    >>>
+    >>> # Progressbar usage:
+    >>> for i in range(100):
+    >>>     window.print_progress(i + 1, 100)
+    >>>     time.sleep(0.1)
+    >>>
+    >>> # Gettig input from the user:
+    >>> name: str = window.input('Enter your name: ')
+    >>> window.print('Hello, ' + name + '!')
+    >>> # Run these lines to see the messages in the window
+    >>>
+
+    :param name: Optional[str]: The name of the logger
+    :param level: Union[int, str] = logging.NOTSET: The logging level of the logger
+    :param show_time: bool = True: Show the time in the log
+    :param show_level: bool = True: Show the level of logging in the log
+    :param fmt: Optional[str]: Custom formatting for the logger - overrides the default(show_time & show_level)
+    :param datefmt: str = "%H:%M:%S": Custom date-time formatting for the logger
+    :param style: str = '%': Use a style parameter of '%', '{' or '$' to specify that you want to use one of %-formatting,
+        :meth:`str.format` (``{}``) formatting or :class:`string.Template` formatting in your format string.
+    :param colorize_time_and_level: bool = True: Colorizes the time and level using the default colors
+    :param handle_carriage_return: bool = True: Adds a line of space characters to remove any text before the CR
+    :param handle_new_line: bool = True: Places the NewLine characters at the beginning of the text before everything else
+    :param override: bool = True: Overrides the logger attributes even if it already exists
+    :param level_names: Dict[int, str] = None: You can specify custom level names.
+    :param width: int = 80: The width of the window
+    :param height: int = 20: The height of the window
+    :param allow_shell: bool = False: Allow the user to use the shell
+    :return: log21.LoggingWindow
+    """
+    if not isinstance(name, str):
+        raise TypeError('A logger name must be a string')
+    logging_window = None
+    if name:
+        logging_window = _manager.getLogger(name)
+    if (not logging_window) or override:
+        logging_window = LoggingWindow(name, level=level, width=width, height=height, allow_shell=allow_shell)
+        formatter = _prepare_formatter(fmt, style, datefmt, show_level, show_time, colorize_time_and_level, level_names)
+
+        # Defines the handler
+        handler = LoggingWindowHandler(logging_window, handle_carriage_return=handle_carriage_return,
+                                       handle_new_line=handle_new_line)
+        handler.setFormatter(formatter)
+        logging_window.addHandler(handler)
+        _manager.addLogger(name, logging_window)
+    return logging_window
+
+
 getLogger = get_logger
 
 
@@ -149,7 +237,9 @@ tprint = tree_print
 root = Logger('root-logger', INFO)
 
 
-def basic_config(**kwargs):
+def basic_config(force: bool = False, encoding: str = None, errors: str = 'backslashreplace', handlers=None,
+                 stream=None, filename=None, filemode: str = 'a', date_format: str = "%H:%M:%S", style: str = '%',
+                 format_: str = None, level: int = None):
     """
     Do basic configuration for the logging system.
 
@@ -201,52 +291,42 @@ def basic_config(**kwargs):
     using sys.stdout or sys.stderr), whereas FileHandler closes its stream
     when the handler is closed.
     """
-    force = kwargs.pop('force', False)
-    encoding = kwargs.pop('encoding', None)
-    errors = kwargs.pop('errors', 'backslashreplace')
     if force:
         for handler in root.handlers[:]:
             root.removeHandler(handler)
             handler.close()
     if len(root.handlers) == 0:
-        handlers = kwargs.pop("handlers", None)
         if handlers is None:
-            if "stream" in kwargs and "filename" in kwargs:
+            if stream and filename:
                 raise ValueError("'stream' and 'filename' should not be specified together")
         else:
-            if "stream" in kwargs or "filename" in kwargs:
+            if stream or filename:
                 raise ValueError("'stream' or 'filename' should not be specified together with 'handlers'")
         if handlers is None:
-            filename = kwargs.pop("filename", None)
-            mode = kwargs.pop("filemode", 'a')
             if filename:
-                if 'b' in mode:
+                if 'b' in filemode:
                     errors = None
                 else:
                     encoding = _io.text_encoding(encoding)
-                handler = DecolorizingFileHandler(filename, mode, encoding=encoding, errors=errors)
+                handler = DecolorizingFileHandler(filename, filemode, encoding=encoding, errors=errors)
             else:
-                stream = kwargs.pop("stream", None)
                 handler = ColorizingStreamHandler(stream=stream)
             handlers = [handler]
-        date_format = kwargs.pop("datefmt", "%H:%M:%S")
-        style = kwargs.pop("style", '%')
         if style not in '%{$':
             raise ValueError('Style must be one of: %, {, $')
-        format_ = kwargs.pop("format", {'%': '[%(asctime)s] [%(levelname)s] %(message)s',
-                                        '{': '[{asctime}] [{levelname}] {message}',
-                                        '$': '[${asctime}] [${levelname}] ${message}'}[style])
+        if not format_:
+            format_ = {'%': '[%(asctime)s] [%(levelname)s] %(message)s',
+                       '{': '[{asctime}] [{levelname}] {message}',
+                       '$': '[${asctime}] [${levelname}] ${message}'}[style]
+        else:
+            format_ = format_
         formatter = ColorizingFormatter(format_, date_format, style=style)
         for handler in handlers:
             if handler.formatter is None:
                 handler.setFormatter(formatter)
             root.addHandler(handler)
-        level = kwargs.pop("level", None)
         if level is not None:
             root.setLevel(level)
-        if kwargs:
-            keys = ', '.join(kwargs.keys())
-            raise ValueError('Unrecognised argument(s): %s' % keys)
 
 
 basicConfig = basic_config
@@ -329,3 +409,14 @@ def log(level, *msg, args=(), **kwargs):
     if len(root.handlers) == 0:
         basic_config()
     root.log(level, *msg, args=args, **kwargs)
+
+
+def progress_bar(progress: float, total: float, width: int = None, prefix: str = '|', suffix: str = '|',
+                 show_percentage: bool = True):
+    """
+    Print a progress bar to the console.
+    """
+
+    bar = ProgressBar(width=width, prefix=prefix, suffix=suffix, show_percentage=show_percentage)
+
+    print(bar.get_bar(progress, total))
